require_relative "file_system"
require_relative "error/missing_torque_info_file_error"
require 'yaml'

class Torque

  ##
  # Parses the raw contents of the torque info file
  # Stores each of the fields in .torqueinfo.yaml as publicly accessible fields
  class TorqueInfoParser

    ##
    # The email_address field (String)
    attr_reader :email_address

    ##
    # The email_password field (String)
    attr_reader :email_password

    ##
    # The email_to field (String or Array)
    attr_reader :email_to

    ##
    # The format string to use for the project
    attr_reader :format

    ##
    # The project field (Fixnum)
    attr_reader :project

    ##
    # The output_dir field (String)
    attr_reader :output_dir

    ##
    # The token field (String)
    attr_reader :token

    ##
    # @param file_path The path to the .torqueinfo.yaml file
    # @param file_system An instance of the FileSystem class
    #
    # Creates a new parser. Does not parse the file
    def initialize(file_path = "./.torqueinfo.yaml", file_system = FileSystem.new)

      @file_path = file_path
      @fs = file_system
    end

    ##
    # @return The TorqueInfoParser (self)
    #
    # Parses the file, storing the results as public instnance fields
    # Stores each field with no corresponding value in the file as 'nil'
    def parse

      if !@fs.path_exist?(@file_path)
        directory = File.expand_path(File.dirname(@file_path))
        raise MissingTorqueInfoFileError.new "'#{directory}' is not configured for Torque"
      end

      torque_info_hash = {}
      begin
        torque_info_hash = YAML::load(@fs.file_read(@file_path)) || {}
      rescue Psych::SyntaxError
        # TODO Implement custom error, raise/rescue paths to catch this problem
      end

      @email_address = torque_info_hash["email_address"]
      @email_password = torque_info_hash["email_password"]
      @email_to = torque_info_hash["email_to"]
      @format = torque_info_hash["format"]
      @project = torque_info_hash["project"]
      @output_dir = torque_info_hash["output_dir"]
      @token = torque_info_hash["token"]

      self
    end

    ##
    # @param field The field to set
    # @param value The value to set the field to 
    def set(field, value)
      file_string = @fs.file_read(@file_path)
      new_file_string = set_string(field, value, file_string)
      @fs.file_write(@file_path, new_file_string)
    end

    ##
    # @param field The field to add to
    # @param values A list of values to add to the field
    def add(field, values)
      file_string = @fs.file_read(@file_path)
      new_file_string = add_string(field, values, file_string)
      @fs.file_write(@file_path, new_file_string)
    end

    ##
    # @param field The field to add to
    # @param values A list of values to add to the field
    #
    # @return A list of all values that were added (ie that were not duplicates)
    def add_no_duplicates(field, values)
      file_string = @fs.file_read(@file_path)
      values_copy = values.clone

      new_file_string = add_no_duplicates_string(field, values_copy, file_string) 
      @fs.file_write(@file_path, new_file_string)
      values_copy
    end

    ##
    # @param field The field to remove from
    # @param values A list of values to remove from the field
    #
    # @return A list of the values that were removed (ie that could be found)
    def rm(field, values)
      file_string = @fs.file_read(@file_path)
      values_copy = values.clone
      
      new_file_string = rm_string(field, values_copy, file_string)
      @fs.file_write(@file_path, new_file_string)
      values_copy
    end


    private

    # Util method. Changes the first array to the second array in place
    def set_array_in_place(array1, array2)
      array1.delete_if {true}
      array1.concat(array2)
    end

    # Util method. Converts a yaml field to an applicable array
    def to_array(value)
      if    value.is_a? Array;    value
      elsif value.is_a? String;   [value]
      elsif value.is_a? NilClass; []
      else; []
      end
    end

    # Alters the string contents of a file, setting the specified field to the specified value
    def set_string(field, value, file_string)

      file_yaml = YAML.load(file_string) || {}
      file_yaml[field] = value
      file_yaml.to_yaml

    end

    # Alters the string contents of a file, adding all elements of 'values' to field 'field'
    # Returns the new string contents
    def add_string(field, values, file_string)

      return file_string if values.empty?

      file_yaml = YAML.load(file_string) || {}
      file_yaml[field] = to_array(file_yaml[field])
      file_yaml[field].concat(values)
      file_yaml.to_yaml

    end

    # Alters the string contents of a file, adding all elements of 'values' to field 'field' ignoring duplicates
    # At finish, 'values' contains all values that were added (were not duplicates)
    # Returns the new string contents
    def add_no_duplicates_string(field, values, file_string)

      return file_string if values.empty?

      file_yaml = YAML.load(file_string) || {}
      file_yaml[field] = to_array(file_yaml[field])
      file_yaml[field].delete(nil)
      
      values.uniq!

      index = 0
      while index < values.length
        value = values[index]

        if file_yaml[field].member? value
          values.delete_at(index)
          index-=1
        else
          file_yaml[field] << value
        end

        index+=1
      end

      file_yaml.to_yaml

    end

    # Alters the string contents of a file so that value is added to a sequence belonging to field
    # At finish, 'values' contains all values that were removed (could be found)
    # Returns the new string contents
    def rm_string(field, values, file_string)

      return file_string if values.empty?

      file_yaml = YAML.load(file_string) || {}
      
      # The field does not exist
      if file_yaml[field].is_a? NilClass
        set_array_in_place(values, [])
        return file_string
        
      # The field is a single value
      elsif file_yaml[field].is_a? String
        if values.member? file_yaml[field]
          set_array_in_place(values, [ file_yaml[field] ])
          file_yaml.delete(field)
        else
          set_array_in_place(values, [])
        end

      # The field is a sequence
      else
        index = 0;
        while index < values.length
          value = values[index]
          if file_yaml[field].member? value
            file_yaml[field].delete(value)
          else
            values.delete_at(index)
            index -= 1
          end
          index += 1
        end

        file_yaml.delete(field) if file_yaml[field].empty?
      end

      file_yaml.to_yaml
    end

  end
end
